package com.yihu.wlyy.util;

import com.yihu.wlyy.util.httpUtil.HttpResponse;
import com.yihu.wlyy.util.httpUtil.HttpUtils;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.apache.commons.net.ftp.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by zdm on 2019/5/7.
 */
@Component
public class FtpFilesUtil {
    static Logger Log = Logger.getLogger(FtpFilesUtil.class);
    private FTPClient ftpClient;
    private static String ENCODING = "GBK";
    FTPClientConfig ftpConfig = new FTPClientConfig("UNIX");
    @Value("${wlyy.url}")
    private String wlyyUrl;
    @Value("${file.default_path}")
    private String fileDefaultPath;


    /**
     * 连接到ftp
     * @param address ip
     * @param port 端口
     * @param name 用户名
     * @param passwd 密码
     * @return
     */
    public boolean connectServer(String address,int port,String name,String passwd) {
        boolean flag = false;
        try {
            ftpClient = new FTPClient();
            ftpConfig.setServerLanguageCode("ISO-8859-1");
            ftpClient.connect(address, port);
            if(ftpClient.login(name,passwd)){
                if (FTPReply.isPositiveCompletion(ftpClient.sendCommand("OPTS UTF8", "ON"))) {
                    ENCODING = "UTF-8";
                }
                ftpClient.setControlEncoding(ENCODING);
                ftpClient.enterLocalPassiveMode();
                ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
                ftpClient.setFileTransferMode(FTP.STREAM_TRANSFER_MODE);
                ftpClient.setDataTimeout(120000);
            }
            int reply = ftpClient.getReplyCode();
            if (FTPReply.isPositiveCompletion(reply)) {
                flag = true;
                Log.info("FTP connect success!");
            } else {
                Log.warn("FTP refused to connect!");
                ftpClient.disconnect();
            }
        } catch (Exception e) {
            Log.error("Failed to login ftp " + address + ":" + port, e);
        }
        return flag;
    }

    /**
     * 判断服务器上是否存在该文件
     * @param path
     * @return
     * @throws IOException
     */
    public boolean existFile(String path) throws IOException {
        boolean flag = false;
        FTPFile[] ftpFileArr = ftpClient.listFiles(path);
        if (ftpFileArr.length > 0) {
            flag = true;
        }else {
            Log.info("file ‘"+path+ "' does not exist ! ");
        }
        return flag;
    }

    /**
     * 上传文件到服务器
     * @param remoteFilePath 远程文件路径： /data/rec2/2019/0507/08/52.mp3
     * @param uploadFile 待上传文件： D:\testFile\52.mp3
     * @return
     * @throws IOException
     */
    public boolean uploadFile(String remoteFilePath, File uploadFile) throws IOException {
        boolean flag = false;
        InputStream input = null;
        try {
            input = new FileInputStream(uploadFile);
            String remote = new String(remoteFilePath.getBytes(ENCODING), "ISO-8859-1");
            if (ftpClient.appendFile(remote, input)) {
                flag = true;
            }
        } finally {
            input.close();
        }
        Log.info("push file (" + uploadFile.getCanonicalPath() + ") => " + (flag ? "SUCCESS" : "FAILED"));
        return flag;
    }

    /**
     * 关闭服务器连接
     */
    public void closeConnect() {
        try {
            if (ftpClient != null) {
                ftpClient.logout();
                ftpClient.disconnect();
                Log.info("FTP connect closed!");
            }
        } catch (Exception e) {
            Log.info("Failed to close FTP connection!");
            e.printStackTrace();
        }
    }


    /**
     * 从FTP下载文件到本地
     * @param ftpClient     已经登陆成功的FTPClient
     * @param ftpFilePath   FTP上的目标文件路径
     * @param localFilePath 下载到本地的文件路径
     */
    public void downloadFileFromFTP(FTPClient ftpClient, String ftpFilePath, String localFilePath,String fileName) {
        InputStream is = null;
        FileOutputStream fos = null;
        try {
            // 获取ftp上的文件
            is = ftpClient.retrieveFileStream(ftpFilePath);
            fos = new FileOutputStream(new File(localFilePath+fileName));
            // 文件读取方式一
            int i;
            byte[] bytes = new byte[1024];
            while ((i = is.read(bytes)) != -1) {
                fos.write(bytes, 0, i);
            }
            ftpClient.completePendingCommand();
            Log.info("FTP文件下载成功！");
        } catch (Exception e) {
            Log.error("FTP文件下载失败！", e);
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 递归遍历出目录下面所有文件，下载到本地
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public JSONArray getFileListByPathTest(String url,int port,String user,String pwd,String pathName) throws Exception {
        JSONArray jsonArray = new JSONArray();
        //连接ftp服务器
        connectServer(url,port,user,pwd);
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            ftpClient.enterLocalPassiveMode();
            ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = ftpClient.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        downloadFileFromFTP(ftpClient, pathName+files[i].getName(),"D:\\testFile\\", files[i].getName());

                    }
                }
            }
        }
        //关闭连接
        closeConnect();
        return jsonArray;
    }
    //========================================================文件获取并上传=================================================================================================

    /**
     * 递归遍历出目录下面所有文件
     * @param url ftp_ip
     * @param port ftp端口
     * @param user ftp登录用户名
     * @param pwd ftp密码
     * @throws Exception
     */
    public void uploadFtpFileJobByDate(String url, int port, String user, String pwd) throws Exception {
        //连接ftp服务器
        connectServer(url, port, user, pwd);
        //获取今天日期 拼凑录音文件在ftp上的路径
        String year=DateUtil.getNowYear()+"";
        String date=   DateUtil.dateToStr(DateUtil.getNowDate(),DateUtil.MMDD);
        String pathName=fileDefaultPath+year+"/"+date+"/";
        ergodicFiles(pathName);
        //关闭连接
        closeConnect();
    }

    public void ergodicFiles(String pathName) throws Exception{
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            this.ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            this.ftpClient.enterLocalPassiveMode();
            this.ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = this.ftpClient.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        inputStreamUpload(this.ftpClient, pathName , files[i].getName());
                        Log.info("DATE：" + DateUtil.getStringDate() + ";FILES_NAME=" + files[i].getName());
                    }else{
                        String time=files[i].getName();
                        Log.info("filesPathName===================="+pathName+time+"/");
                        ergodicFiles(pathName+time+"/");
                    }
                }
            }
        }
    }

    /**
     * 递归遍历出目录下面所有文件
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public void uploadFtpFileByPath(String url, int port, String user, String pwd, String pathName) throws Exception {
        //连接ftp服务器
        connectServer(url, port, user, pwd);
        //递归查找文件，并上传
        ergodicFiles(pathName);
        //关闭连接
        closeConnect();
    }

    /**
     * 测试某路径递归遍历出目录下面所有文件
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public JSONArray getFileListByPathTest1(String url,int port,String user,String pwd,String pathName) throws Exception {
        JSONArray jsonArray = new JSONArray();
        //连接ftp服务器
        connectServer(url,port,user,pwd);
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //设置访问被动模式
            ftpClient.enterLocalPassiveMode();
            ftpClient.setRemoteVerificationEnabled(false);
            //更换目录到当前目录
            ftpClient.changeWorkingDirectory(pathName);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = ftpClient.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        int size = (int) files[i].getSize();
                        Log.info("fileSize=========" + size);
                        inputStreamUpload( ftpClient, pathName,files[i].getName());
                    }
                }
            }
        }
        //关闭连接
        closeConnect();
        return jsonArray;
    }

    //====================================================http发送文件流===============================================================================
    public void inputStreamUpload(FTPClient ftpClient, String ftpFilePath,String fileName) throws Exception{
        //获取ftp的文件流
        InputStream inputStream = new BufferedInputStream(ftpClient.retrieveFileStream(ftpFilePath+fileName));
        //创建HttpClient对象
        CloseableHttpClient client = HttpClients.createDefault();
        //请求远程上传数据接口wlyyUrl + "upload/ftpVoiceUpload"
        HttpPost post = new HttpPost(wlyyUrl+"upload/fastDFSVoiceByRequest");
        try {
            //文件路径请换成自己的
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
            //第一个参数为 相当于 Form表单提交的file框的name值、第二个参数就是我们要发送的InputStream对象了、第三个参数是文件名
            builder.addBinaryBody("file", inputStream, ContentType.create("multipart/form-data"), fileName);
            //构建请求参数 普通表单项
            StringBody stringBody = new StringBody("12",ContentType.MULTIPART_FORM_DATA);
            builder.addPart("file",stringBody);
            HttpEntity entity = builder.build();
            post.setEntity(entity);
            //发送请求
            CloseableHttpResponse response = client.execute(post);
            entity = response.getEntity();
            if (entity != null) {
                String result = EntityUtils.toString(response.getEntity(), "utf-8");
                //控制台输出请求结果
                Log.info(result);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            if(inputStream != null){
                try {
                    inputStream.close();
                    ftpClient.completePendingCommand();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }



}
